]> andersk Git - openssh.git/blobdiff - authfd.c
- (djm) Merge OpenBSD changes:
[openssh.git] / authfd.c
index d06cc536c9dd593e611e84d5bf2095d0aebc7994..9036a8d899ab619e251ad421d7d125108c941ab1 100644 (file)
--- a/authfd.c
+++ b/authfd.c
@@ -35,7 +35,7 @@
  */
 
 #include "includes.h"
-RCSID("$OpenBSD: authfd.c,v 1.29 2000/10/09 21:51:00 markus Exp $");
+RCSID("$OpenBSD: authfd.c,v 1.30 2000/11/12 19:50:37 markus Exp $");
 
 #include "ssh.h"
 #include "rsa.h"
@@ -50,7 +50,6 @@ RCSID("$OpenBSD: authfd.c,v 1.29 2000/10/09 21:51:00 markus Exp $");
 #include "key.h"
 #include "authfd.h"
 #include "kex.h"
-#include "dsa.h"
 #include "compat.h"
 
 /* helper */
@@ -211,8 +210,8 @@ ssh_close_authentication_connection(AuthenticationConnection *auth)
  * Returns the first authentication identity held by the agent.
  */
 
-Key *
-ssh_get_first_identity(AuthenticationConnection *auth, char **comment, int version)
+int
+ssh_get_num_identities(AuthenticationConnection *auth, int version)
 {
        int type, code1 = 0, code2 = 0;
        Buffer request;
@@ -227,7 +226,7 @@ ssh_get_first_identity(AuthenticationConnection *auth, char **comment, int versi
                code2 = SSH2_AGENT_IDENTITIES_ANSWER;
                break;
        default:
-               return NULL;
+               return 0;
        }
 
        /*
@@ -240,14 +239,14 @@ ssh_get_first_identity(AuthenticationConnection *auth, char **comment, int versi
        buffer_clear(&auth->identities);
        if (ssh_request_reply(auth, &request, &auth->identities) == 0) {
                buffer_free(&request);
-               return NULL;
+               return 0;
        }
        buffer_free(&request);
 
        /* Get message type, and verify that we got a proper answer. */
        type = buffer_get_char(&auth->identities);
        if (agent_failed(type)) {
-               return NULL;
+               return 0;
        } else if (type != code2) {
                fatal("Bad authentication reply message type: %d", type);
        }
@@ -258,8 +257,16 @@ ssh_get_first_identity(AuthenticationConnection *auth, char **comment, int versi
                fatal("Too many identities in authentication reply: %d\n",
                    auth->howmany);
 
-       /* Return the first entry (if any). */
-       return ssh_get_next_identity(auth, comment, version);
+       return auth->howmany;
+}
+
+Key *
+ssh_get_first_identity(AuthenticationConnection *auth, char **comment, int version)
+{
+       /* get number of identities and return the first entry (if any). */
+       if (ssh_get_num_identities(auth, version) > 0)
+               return ssh_get_next_identity(auth, comment, version);
+       return NULL;
 }
 
 Key *
@@ -280,7 +287,7 @@ ssh_get_next_identity(AuthenticationConnection *auth, char **comment, int versio
         */
        switch(version){
        case 1:
-               key = key_new(KEY_RSA);
+               key = key_new(KEY_RSA1);
                bits = buffer_get_int(&auth->identities);
                buffer_get_bignum(&auth->identities, key->rsa->e);
                buffer_get_bignum(&auth->identities, key->rsa->n);
@@ -292,7 +299,7 @@ ssh_get_next_identity(AuthenticationConnection *auth, char **comment, int versio
        case 2:
                blob = buffer_get_string(&auth->identities, &blen);
                *comment = buffer_get_string(&auth->identities, NULL);
-               key = dsa_key_from_blob(blob, blen);
+               key = key_from_blob(blob, blen);
                xfree(blob);
                break;
        default:
@@ -324,7 +331,7 @@ ssh_decrypt_challenge(AuthenticationConnection *auth,
        int i;
        int type;
 
-       if (key->type != KEY_RSA)
+       if (key->type != KEY_RSA1)
                return 0;
        if (response_type == 0) {
                log("Compatibility with ssh protocol version 1.0 no longer supported.");
@@ -376,7 +383,7 @@ ssh_agent_sign(AuthenticationConnection *auth,
        int type, flags = 0;
        int ret = -1;
 
-       if (dsa_make_key_blob(key, &blob, &blen) == 0)
+       if (key_to_blob(key, &blob, &blen) == 0)
                return -1;
 
        if (datafellows & SSH_BUG_SIGBLOB)
@@ -409,7 +416,7 @@ ssh_agent_sign(AuthenticationConnection *auth,
 /* Encode key for a message to the agent. */
 
 void
-ssh_encode_identity_rsa(Buffer *b, RSA *key, const char *comment)
+ssh_encode_identity_rsa1(Buffer *b, RSA *key, const char *comment)
 {
        buffer_clear(b);
        buffer_put_char(b, SSH_AGENTC_ADD_RSA_IDENTITY);
@@ -425,17 +432,29 @@ ssh_encode_identity_rsa(Buffer *b, RSA *key, const char *comment)
 }
 
 void
-ssh_encode_identity_dsa(Buffer *b, DSA *key, const char *comment)
+ssh_encode_identity_ssh2(Buffer *b, Key *key, const char *comment)
 {
        buffer_clear(b);
        buffer_put_char(b, SSH2_AGENTC_ADD_IDENTITY);
-       buffer_put_cstring(b, KEX_DSS);
-       buffer_put_bignum2(b, key->p);
-       buffer_put_bignum2(b, key->q);
-       buffer_put_bignum2(b, key->g);
-       buffer_put_bignum2(b, key->pub_key);
-       buffer_put_bignum2(b, key->priv_key);
-       buffer_put_string(b, comment, strlen(comment));
+       buffer_put_cstring(b, key_ssh_name(key));
+       switch(key->type){
+       case KEY_RSA:
+               buffer_put_bignum2(b, key->rsa->n);
+               buffer_put_bignum2(b, key->rsa->e);
+               buffer_put_bignum2(b, key->rsa->d);
+               buffer_put_bignum2(b, key->rsa->iqmp);
+               buffer_put_bignum2(b, key->rsa->p);
+               buffer_put_bignum2(b, key->rsa->q);
+               break;
+       case KEY_DSA:
+               buffer_put_bignum2(b, key->dsa->p);
+               buffer_put_bignum2(b, key->dsa->q);
+               buffer_put_bignum2(b, key->dsa->g);
+               buffer_put_bignum2(b, key->dsa->pub_key);
+               buffer_put_bignum2(b, key->dsa->priv_key);
+               break;
+       }
+       buffer_put_cstring(b, comment);
 }
 
 /*
@@ -452,11 +471,12 @@ ssh_add_identity(AuthenticationConnection *auth, Key *key, const char *comment)
        buffer_init(&msg);
 
        switch (key->type) {
-       case KEY_RSA:
-               ssh_encode_identity_rsa(&msg, key->rsa, comment);
+       case KEY_RSA1:
+               ssh_encode_identity_rsa1(&msg, key->rsa, comment);
                break;
+       case KEY_RSA:
        case KEY_DSA:
-               ssh_encode_identity_dsa(&msg, key->dsa, comment);
+               ssh_encode_identity_ssh2(&msg, key, comment);
                break;
        default:
                buffer_free(&msg);
@@ -487,13 +507,13 @@ ssh_remove_identity(AuthenticationConnection *auth, Key *key)
 
        buffer_init(&msg);
 
-       if (key->type == KEY_RSA) {
+       if (key->type == KEY_RSA1) {
                buffer_put_char(&msg, SSH_AGENTC_REMOVE_RSA_IDENTITY);
                buffer_put_int(&msg, BN_num_bits(key->rsa->n));
                buffer_put_bignum(&msg, key->rsa->e);
                buffer_put_bignum(&msg, key->rsa->n);
-       } else if (key->type == KEY_DSA) {
-               dsa_make_key_blob(key, &blob, &blen);
+       } else if (key->type == KEY_DSA || key->type == KEY_RSA) {
+               key_to_blob(key, &blob, &blen);
                buffer_put_char(&msg, SSH2_AGENTC_REMOVE_IDENTITY);
                buffer_put_string(&msg, blob, blen);
                xfree(blob);
This page took 0.075138 seconds and 4 git commands to generate.